#!/usr/bin/perl
# brookbench.br generator
my @functions = ();

print_header();
print_kernel_ops(16, "float");
print_kernel_ops(16, "float4");
print_kernel_ops(64, "float");
print_kernel_ops(64, "float4");
#print_kernel_ops(256, "float");
#print_kernel_ops(256, "float4");
#print_kernel_ops(512, "float");
#print_kernel_ops(512, "float4");


push(@functions, print_stream_1way("download",  "float",  512, 512));
push(@functions, print_stream_1way("download",  "float4", 512, 512));
push(@functions, print_stream_1way("download",  "float",  1024, 1024));
push(@functions, print_stream_1way("download",  "float4", 1024, 1024));

push(@functions, print_stream_1way("readback", "float",  512, 512));
push(@functions, print_stream_1way("readback", "float4", 512, 512));
push(@functions, print_stream_1way("readback", "float",  1024, 1024));
push(@functions, print_stream_1way("readback", "float4", 1024, 1024));

push(@functions, print_stream_opb("copy", "float",  512, 512));
push(@functions, print_stream_opb("copy", "float4", 512, 512));
push(@functions, print_stream_opb("copy", "float",  1024, 1024));
push(@functions, print_stream_opb("copy", "float4", 1024, 1024));

push(@functions, print_stream_dor("copy", "float",  512,  512));
push(@functions, print_stream_dor("copy", "float4", 512,  512));
push(@functions, print_stream_dor("16ops", "float",  512,  512));
push(@functions, print_stream_dor("16ops", "float4", 512,  512));

push(@functions, print_stream_do("1ops",  1,    "float",  512, 512));
push(@functions, print_stream_do("1ops",  1*4,  "float4", 512, 512));
push(@functions, print_stream_do("16ops", 16,   "float",  512, 512));
push(@functions, print_stream_do("16ops", 16*4, "float4", 512, 512));
push(@functions, print_stream_do("64ops", 64,   "float",  512, 512));
push(@functions, print_stream_do("64ops", 64*4, "float4", 512, 512));

#push(@functions, print_stream_dor("copy", "float",  512,  512));
#push(@functions, print_stream_dor("copy", "float4", 512,  512));
#push(@functions, print_stream_dor("copy", "float",  1024, 1024));
#push(@functions, print_stream_dor("copy", "float4", 1024, 1024));


push(@functions, print_stream_op("1ops",  1,    "float",  1024, 1024));
push(@functions, print_stream_op("1ops",  1*4,  "float4", 1024, 1024));
push(@functions, print_stream_op("16ops", 16,   "float",  1024, 1024));
push(@functions, print_stream_op("16ops", 16*4, "float4", 1024, 1024));
push(@functions, print_stream_op("64ops", 64,   "float",  1024, 1024));
push(@functions, print_stream_op("64ops", 64*4, "float4", 1024, 1024));


#push(@functions, print_stream_dor("1ops", "float4", 256, 256));
#push(@functions, print_stream_dor("16ops", "float4", 256, 256));




print_footer();



#print_stream_rcw();


sub print_header
{
	print <<EOM;

void timer_reset();
void timer_stop_w_bandwidth(const char* name, int op_size, int loops);
void timer_stop_w_gflops(const char* name, int op_size, int loops);

#define LOOP	1024

// statics ////////////////////////////////////////////////////////////////////////////////////////
static float* float_in;
static float4* float4_in;
static float* float_out;
static float4* float4_out;

// kernels ////////////////////////////////////////////////////////////////////////////////////////

kernel void k_copy_float(float stream_in<>, out float stream_out<>)
{
	stream_out = stream_in;
}

kernel void k_copy_float4(float4 stream_in<>, out float4 stream_out<>)
{
	stream_out = stream_in;
}

kernel void k_1ops_float(float stream_in<>, out float stream_out<>)
{
	stream_out = stream_in * 2.0;
}

kernel void k_1ops_float4(float4 stream_in<>, out float4 stream_out<>)
{
	stream_out = stream_in * 2.0;
}

EOM
}

sub print_kernel_ops
{
	my ($operations, $type) = @_;
	
	print "kernel void k_" . $operations . "ops_$type($type stream_in<>, out $type stream_out<>)\n";
	print "{\n";
	print "\tstream_out = stream_in;\n";

	for (my $i = 0; $i < $operations / 2; $i++)
	{
		print "\tstream_out += stream_in * stream_out;\n";

	}
	print "}\n\n";
}

sub print_stream_1way
{
	my ($bn, $type, $width, $height) = @_;

#	my $width = "1024";
#	my $height = "1024";
#	my $type = "float";
#	my $bn = "read";

	my $fn = "way1_$bn" . '_' . $width . 'x' . $height . '_' . "$type()";
	my $sn = sprintf("%-10s(%-6s:%4dx%4d)", $bn, $type, $width, $height);

	print "static void $fn\n";
	print "{\n";
	print "\t$type tex1<$width, $height>;\n";
	print "\tint i;\n";
	print "\ttimer_reset();\n";
	print "\tfor (i = 0; i < LOOP; i++)\n";
	print "\t{\n";

	if ($bn eq "download")
	{
		print "\t\tstreamRead(tex1, $type" . "_in);\n";
	}
	else
	{
		print "\t\tstreamWrite(tex1, $type" . "_out);\n";
	}
	
	print "\t}\n";
	print "\ttimer_stop_w_bandwidth(\"$sn\", $width*$height*sizeof($type), LOOP);\n";
	print "}\n\n";

	return $fn;
}

sub print_stream_dor
{
	my ($bn, $type, $width, $height) = @_;

	my $fn = "dor_$bn" . '_' . $width . 'x' . $height . '_' . "$type()";
	my $sn = sprintf("dor_%-6s(%-6s:%4dx%4d)", $bn, $type, $width, $height);

	print "static void $fn\n";
	print "{\n";
	print "\t$type tex1<$width, $height>;\n";
	print "\t$type tex2<$width, $height>;\n";
	print "\tint i;\n";
	print "\ttimer_reset();\n";
	print "\tfor (i = 0; i < LOOP; i++)\n";
	print "\t{\n";
	print "\t\tstreamRead(tex1, $type" . "_in);\n";
	print "\t\tk_$bn" . "_$type(tex1, tex2);\n";
	print "\t\tstreamWrite(tex2, $type" . "_out);\n";
	print "\t}\n";
	print "\ttimer_stop_w_bandwidth(\"$sn\", $width*$height*sizeof($type), LOOP);\n";
	print "}\n\n";

	return $fn;
}

sub print_stream_do
{
	my ($bn, $ops, $type, $width, $height) = @_;

	my $fn = "do_$bn" . '_' . $width . 'x' . $height . '_' . "$type()";
	my $sn = sprintf("do__%-6s(%-6s:%4dx%4d)", $bn, $type, $width, $height);

	print "static void $fn\n";
	print "{\n";
	print "\t$type tex1<$width, $height>;\n";
	print "\t$type tex2<$width, $height>;\n";
	print "\tint i;\n";
	print "\ttimer_reset();\n";
	print "\tfor (i = 0; i < LOOP; i++)\n";
	print "\t{\n";
	print "\t\tstreamRead(tex1, $type" . "_in);\n";
	print "\t\tk_$bn" . "_$type(tex1, tex2);\n";
	print "\t}\n";
	print "\ttimer_stop_w_gflops(\"$sn\", $width*$height*$ops, LOOP);\n";
	print "}\n\n";

	return $fn;
}

sub print_stream_op
{
	my ($bn, $ops, $type, $width, $height) = @_;

	my $fn = "op_$bn" . '_' . $width . 'x' . $height . '_' . "$type()";
	my $sn = sprintf("op__%-6s(%-6s:%4dx%4d)", $bn, $type, $width, $height);

	print "static void $fn\n";
	print "{\n";
	print "\t$type tex1<$width, $height>;\n";
	print "\t$type tex2<$width, $height>;\n";
	print "\tint i;\n";
	print "\tstreamRead(tex1, $type" . "_in);\n";
	print "\ttimer_reset();\n";
	print "\tfor (i = 0; i < LOOP; i++)\n";
	print "\t{\n";
	print "\t\tk_$bn" . "_$type(tex1, tex2);\n";
	print "\t}\n";
	print "\ttimer_stop_w_gflops(\"$sn\", $width*$height*$ops, LOOP);\n";
	print "\tstreamWrite(tex2, $type" . "_out);\n";
	print "}\n\n";

	return $fn;
}

sub print_stream_opb
{
	my ($bn, $type, $width, $height) = @_;

	my $fn = "opb_$bn" . '_' . $width . 'x' . $height . '_' . "$type()";
	my $sn = sprintf("opb_%-6s(%-6s:%4dx%4d)", $bn, $type, $width, $height);

	print "static void $fn\n";
	print "{\n";
	print "\t$type tex1<$width, $height>;\n";
	print "\t$type tex2<$width, $height>;\n";
	print "\tint i;\n";
	print "\tstreamRead(tex1, $type" . "_in);\n";
	print "\ttimer_reset();\n";
	print "\tfor (i = 0; i < LOOP; i++)\n";
	print "\t{\n";
	print "\t\tk_$bn" . "_$type(tex1, tex2);\n";
	print "\t}\n";
	print "\ttimer_stop_w_bandwidth(\"$sn\", $width*$height*sizeof($type), LOOP);\n";
	print "\tstreamWrite(tex2, $type" . "_out);\n";
	print "}\n\n";

	return $fn;
}


sub print_footer
{
	print <<EOM;

// functions //////////////////////////////////////////////////////////////////////////////////////

void alloc_buffers()
{
	float_in  = (float*) malloc(sizeof(float)  * 1024 * 1024);
	float4_in  = (float4*)malloc(sizeof(float4) * 1024 * 1024);
	float_out = (float*) malloc(sizeof(float)  * 1024 * 1024);
	float4_out = (float4*)malloc(sizeof(float4) * 1024 * 1024);
}

void free_buffers()
{
	free(float4_out);
	free(float_out);
	free(float4_in);
	free(float_in);
}

void test()
{
EOM

	foreach my $f (@functions)
	{
		print "\t$f;\n";
	}

	print <<EOM;
}

EOM
}

