#include <stdio.h>
#include <stdlib.h>
#ifdef WIN32
#include <windows.h>
#else // for linux
#include <time.h>
#include <sys/time.h>
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////
// brookbench.br
void alloc_buffers();
void free_buffers();
void test();

///////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef WIN32

static LARGE_INTEGER t;
static double        f;
static int           freq_init = 0;

void timer_reset()
{
	if (!freq_init) 
	{
		LARGE_INTEGER freq;
		QueryPerformanceFrequency(&freq);
		f = (double) freq.QuadPart;
		freq_init = 1;
	  }
	
	QueryPerformanceCounter(&t);
}

static double elapse_time()
{
	LARGE_INTEGER s;
	QueryPerformanceCounter(&s);
	return ((double)(s.QuadPart - t.QuadPart)) / f;
}

#else // for linux

#define SECOND 1000000.0
static timeval reset_time;
void timer_reset()
{
	gettimeofday(&reset_time,0);
}

static double elapse_time()
{
	double current, reset;

	static struct timeval c;
	gettimeofday(&c,0);
 
 	current = c.tv_sec * SECOND + c.tv_usec;
	reset = reset_time.tv_sec * SECOND + reset_time.tv_usec;

	return (current - reset) / SECOND;
}

#endif

void timer_stop_w_bandwidth(const char* name, int op_size, int loops)
{
	double time = elapse_time();
	double gb = (double)op_size * (double)loops / 1024.0 / 1024.0 / 1024.0;
	
	printf("%s : %3.3fGiB/sec (%2.2fMiB x %d in %1.3fsec)\n", name, gb/time, op_size/1024.f/1024.f, loops, time);
}

void timer_stop_w_gflops(const char* name, int op_size, int loops)
{
	double time = elapse_time();
	double bgf = (double)op_size * (double)loops / 1024.0 / 1024.0 / 1024.0;
	
	printf("%s : %3.3fBogoGFLOPS (%2.2fMop x %d in %1.3fsec)\n", name, bgf/time, op_size/1024.f/1024.f, loops, time);
}

///////////////////////////////////////////////////////////////////////////////////////////////////
int main()
{
	printf("brookbench by gpgpu.jp ver.0.01 2006/05/01\n\n");

	alloc_buffers();
	test();
	free_buffers();
}

